package cn.fastfun.controller.api;

import cn.fastfun.controller.param.DeviceQueryParam;
import cn.fastfun.service.SysExcelFieldService;
import cn.fastfun.service.entity.AppDevice;
import com.bridge.dto.ApiDTO;
import com.bridge.dto.ApiPageDTO;
import com.bridge.dto.IdParam;
import com.bridge.service.JpaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @author Bridge AutoGen
 */
@Api(tags = {"设备信息"})
@RestController
@RequestMapping("/api/v1/appdevice")
public class ApiAppDeviceController {
    //业务类
    @Resource(name = "appDeviceService")
    JpaService<AppDevice, String> appDeviceService;

    @Resource
    SysExcelFieldService sysExcelFieldService;

    /**
     * 数据保存
     */
    @ApiOperation(value = "保存信息")
    @RequestMapping(value = "save", method = RequestMethod.POST)
    public ApiDTO save(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AppDevice entity) {
        return ApiDTO.ok("保存成功", appDeviceService.save(entity));
    }

    @ApiOperation(value = "查询信息")
    @RequestMapping(value = "get", method = RequestMethod.POST)
    public ApiDTO get(@RequestBody @ApiParam(name = "设备信息id", required = true) IdParam param) {
        return ApiDTO.ok("查询成功！", appDeviceService.get(param.getId()));
    }

    @ApiOperation(value = "删除信息")
    @RequestMapping(value = "delete", method = RequestMethod.POST)
    public ApiDTO delete(@RequestBody @ApiParam(name = "设备信息id", required = true) IdParam param) {
        appDeviceService.delete(Arrays.asList(param.getId().split(",")));
        return ApiDTO.ok("删除成功！");
    }

    @ApiOperation(value = "分页搜索")
    @RequestMapping(value = "pageQuery", method = RequestMethod.POST)
    public ApiPageDTO pageQuery(@RequestBody DeviceQueryParam param) {
        return new ApiPageDTO(null, appDeviceService.findByParam(param));
    }

    @ApiOperation(value = "导入模板下载")
    @GetMapping("excelTemplate")
    public void excelTemplate(HttpServletResponse response, @RequestParam String name) {
        HSSFWorkbook workbook = sysExcelFieldService.getTemplate("invoice");
        sysExcelFieldService.outExcelFile(workbook, response, name);
    }

    @ApiOperation(value = "数据导入")
    @RequestMapping(value = "excelImport", method = RequestMethod.POST)
    public ApiDTO excelImport(MultipartFile file) {
        List<Map<String, Object>> list = sysExcelFieldService.importExcel("invoice", file);
        List<AppDevice> devices = new ArrayList<>();
        list.forEach(p -> devices.add(new AppDevice()));

        appDeviceService.save(devices);
        return ApiDTO.ok();
    }

    @ApiOperation(value = "数据计数")
    @RequestMapping(value = "count", method = RequestMethod.POST)
    public ApiDTO excelImport() {
        Long deviceCount = appDeviceService.getCount();
        return ApiDTO.ok(deviceCount);
    }

}