package cn.fastfun.controller.api;


import cn.fastfun.service.ProductService;
import cn.fastfun.service.entity.Product;
import cn.fastfun.service.entity.QueryParamForProduct;
import cn.fastfun.util.VerifyUtil;
import com.bridge.dto.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;

@Api(tags = "设备类型记录（典表）")
@RestController
@RequestMapping("/api/v1/product")
public class ProductController {

    @Resource
    ProductService productService;

    @ApiOperation(value = "保存SN规则信息")
    @PostMapping("/save")
    public ApiDTO save(@RequestBody @ApiParam(name = "SN规则对象", value = "传入json格式", required = true)Product product) {

        // 针对产品规格这一项进行操作
        if(product.getSnType().equals("产品规格")){

            String[] SnTypeDescList = product.getSnTypeDesc().split(" ");
            List<Product> productSpecification = new ArrayList<>();

            String chemicalSystemCode = product.getSnTypeCode().substring(0,1);
            String chemicalSystemDesc = SnTypeDescList[0];
            String batteryPlatformCode = product.getSnTypeCode().substring(1,2);
            String batteryPlatformDesc = SnTypeDescList[1];
            String capacityCode = product.getSnTypeCode().substring(2,4);
            String capacityDesc = SnTypeDescList[2];

            Product chemicalSystem = new Product();
            chemicalSystem.setSnType("化学体系");
            chemicalSystem.setSnTypeCode(chemicalSystemCode);
            chemicalSystem.setSnTypeDesc(chemicalSystemDesc);
            chemicalSystem.setOperator(product.getOperator());
            String checkSnTypeCodeResultOnChemicalSystem = VerifyUtil.checkSnTypeCode(chemicalSystem);
            if(checkSnTypeCodeResultOnChemicalSystem.equals("passed")){
                Product tmp_1 = productService.save(chemicalSystem);
                productSpecification.add(tmp_1);}
            else{
                return ApiDTO.error(500,checkSnTypeCodeResultOnChemicalSystem);
            }

            Product batteryPlatform = new Product();
            batteryPlatform.setSnType("电池平台");
            batteryPlatform.setSnTypeCode(batteryPlatformCode);
            batteryPlatform.setSnTypeDesc(batteryPlatformDesc);
            batteryPlatform.setOperator(product.getOperator());
            String checkSnTypeCodeResultOnBatteryPlatform = VerifyUtil.checkSnTypeCode(batteryPlatform);
            if(checkSnTypeCodeResultOnBatteryPlatform.equals("passed")){
                Product tmp_2 = productService.save(batteryPlatform);
                productSpecification.add(tmp_2);}
            else{
                return ApiDTO.error(500,checkSnTypeCodeResultOnChemicalSystem);
            }

            Product capacity = new Product();
            capacity.setSnType("容量");
            capacity.setSnTypeCode(capacityCode);
            capacity.setSnTypeDesc(capacityDesc);
            capacity.setOperator(product.getOperator());
            String checkSnTypeCodeResultOnCapacity = VerifyUtil.checkSnTypeCode(capacity);
            if(checkSnTypeCodeResultOnCapacity.equals("passed")){
                Product tmp_3 = productService.save(capacity);
                productSpecification.add(tmp_3);}
            else{
                return ApiDTO.error(500,checkSnTypeCodeResultOnChemicalSystem);
            }


            return ApiDTO.ok("保存成功",productSpecification);
        }


        // 校验输入的SN类型对应的编号
        String checkSnTypeCodeResult = VerifyUtil.checkSnTypeCode(product);

        if(checkSnTypeCodeResult.equals("passed")){
        Product tmp = productService.save(product);
        return ApiDTO.ok("保存成功",tmp);}
        else{
            return ApiDTO.error(500,checkSnTypeCodeResult);
        }
    }

    @ApiOperation(value = "查询SN规则信息")
    @PostMapping("/get")
    public ApiDTO get(@RequestBody @ApiParam(name = "SN规则行对应的id", required = true) IdParam param) {
        return ApiDTO.ok("查询成功！", productService.get(param.getId()));
    }

    @ApiOperation(value = "删除SN规则信息")
    @PostMapping("/delete")
    public ApiDTO delete(@RequestBody @ApiParam(name = "SN规则行对应的id", required = true) IdParam param) {
        Product product = productService.get(param.getId());
        productService.delete(Arrays.asList(param.getId().split(",")));
        return ApiDTO.ok("删除成功！",product);
    }


    @ApiOperation(value = "分页搜索SN规则信息")
    @PostMapping("/pageQuery")
    public ApiDTO pageQuery(@RequestBody QueryParamForProduct param) {

        List<Product> resultByType = productService.findAll(QueryParamExp.eq("snType",param.getType()));
        resultByType.sort(Comparator.comparing(Product::getAddTime).reversed());
        int resultSize = resultByType.size();
        int startIndex = (param.getIndex() - 1) * param.getLength();
        int endIndex = startIndex +  param.getLength();
        List<Product> resultByTypeAndPage;
        try {
            if(endIndex > resultSize){
                endIndex = resultSize;
                resultByTypeAndPage = resultByType.subList(startIndex,endIndex);
            }else{
                resultByTypeAndPage = resultByType.subList(startIndex,endIndex);
            }
        }catch(IllegalArgumentException e){
            return ApiDTO.error("分页索引越界！");
        }
        return ApiDTO.ok("搜索成功！",resultByTypeAndPage);
    }

}
